Imports System.Reflection
Imports System.EnterpriseServices
Imports System.Data
Imports System.Data.SqlClient


Namespace Supermarket

    Public Interface ICreateOrder
        Function Create(ByVal OrderNumber As String, _
                        ByVal SupplierName As String) _
                        As Boolean
        Function AddItems(ByVal OrderNumber As String, _
                        ByVal SKU As String, _
                        ByVal Quantity As Integer) _
                        As Boolean
    End Interface

    <ConstructionEnabled([Default]:="Default Construction String"), _
    Transaction(TransactionOption.Required)> _
    Public Class CreateOrder
        Inherits ServicedComponent
        Implements ICreateOrder


        Public Sub New()

        End Sub

        Public Function Create(ByVal OrderNumber As String, _
                        ByVal SupplierName As String) _
                        As Boolean _
                        Implements ICreateOrder.Create

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim intRowsAffected As Integer

            Try
                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = _
                    "INSERT INTO Orders " & _
                    "( OrderNumber, SupplierName, OrderReceived ) " & _
                    "VALUES ( @OrderNumber, @SupplierName, @OrderReceived )"

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@OrderNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = OrderNumber
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@SupplierName"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = SupplierName
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@OrderReceived"
                    .SqlDbType = SqlDbType.Bit
                    .Direction = ParameterDirection.Input
                    .Value = False
                End With
                objCmd.Parameters.Add(objParam)

                intRowsAffected = objCmd.ExecuteNonQuery()

                Create = True
                ContextUtil.SetComplete()
            Catch E As Exception
                Create = False
                ContextUtil.SetAbort()
            End Try
        End Function

        Public Function AddItems(ByVal OrderNumber As String, _
                        ByVal SKU As String, _
                        ByVal Quantity As Integer) _
                        As Boolean _
                        Implements ICreateOrder.AddItems

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim intMaxLineNumber As Integer
            Dim intRowsAffected As Integer
            Dim objTemp As Object

            Try
                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = "SELECT MAX( LineItemNumber ) " & _
                            "FROM OrderDetails " & _
                            "WHERE OrderNumber = @OrderNumber"
                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@OrderNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = OrderNumber
                End With
                objCmd.Parameters.Add(objParam)

                objTemp = objCmd.ExecuteScalar()
                If TypeOf objTemp Is DBNull Then
                    intMaxLineNumber = 1
                Else
                    intMaxLineNumber = CType(objTemp, Integer)
                    intMaxLineNumber += 1
                End If

                objCmd = objCnn.CreateCommand()
                objCmd.CommandText = "INSERT INTO OrderDetails " & _
                        "( OrderNumber, LineItemNumber, SKU, " & _
                        "QuantityReceived, Quantity ) VALUES " & _
                        "( @OrderNumber, @LineNumber, @SKU, " & _
                        "@QuantityReceived, @Quantity )"
                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@OrderNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = OrderNumber
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@LineNumber"
                    .SqlDbType = SqlDbType.Int
                    .Direction = ParameterDirection.Input
                    .Value = intMaxLineNumber
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@SKU"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = SKU
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@QuantityReceived"
                    .SqlDbType = SqlDbType.Int
                    .Direction = ParameterDirection.Input
                    .Value = 0
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@Quantity"
                    .SqlDbType = SqlDbType.Int
                    .Direction = ParameterDirection.Input
                    .Value = Quantity
                End With
                objCmd.Parameters.Add(objParam)

                intRowsAffected = objCmd.ExecuteNonQuery()

                AddItems = True
                ContextUtil.SetComplete()

            Catch E As Exception
                AddItems = False
                ContextUtil.SetAbort()
            Finally
                objCnn.Close()
            End Try

        End Function

    End Class
End Namespace
