Imports System
Imports System.Reflection
Imports System.EnterpriseServices
Imports System.Data
Imports System.Data.SqlClient


Namespace Supermarket

    Public Interface IUpdateInventory
        Function Update(ByVal BinNumber As String, _
                        ByVal SKU As String, _
                        ByVal Quantity As Integer) _
                        As Boolean
        Function GetStockingLocation(ByVal SKU As String) _
                        As String
    End Interface

    <ConstructionEnabled([Default]:="Default Construction String"), _
    Transaction(TransactionOption.Required)> _
    Public Class UpdateInventory
        Inherits ServicedComponent
        Implements IUpdateInventory

        Public Sub New()

        End Sub

        Public Function GetStockingLocation(ByVal SKU As String) _
                As String _
                Implements IUpdateInventory.GetStockingLocation

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim objTemp As Object

            Try
                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = "SELECT StockingBinNumber " & _
                                    "FROM Product WHERE SKU = @SKU"

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@SKU"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = SKU
                End With
                objCmd.Parameters.Add(objParam)

                objTemp = objCmd.ExecuteScalar()
                If TypeOf objTemp Is DBNull Then
                    GetStockingLocation = ""
                Else
                    GetStockingLocation = _
                        CType(objCmd.ExecuteScalar(), String)
                End If
                ContextUtil.SetComplete()

            Catch E As Exception
                ContextUtil.SetAbort()
                GetStockingLocation = ""
            Finally
                objCnn.Close()
            End Try

        End Function

        Private Function InventoryRecExists(ByVal SKU As String, _
                ByVal StockingBinNumber As String) _
                As Boolean

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim intRowCount As Integer
            Dim objTemp As Object

            Try
                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = "SELECT COUNT(*) FROM Inventory " & _
                        "WHERE SKU = @SKU AND " & _
                        "BinNumber = @StockingBinNumber"
                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@SKU"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = SKU
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@StockingBinNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = StockingBinNumber
                End With
                objCmd.Parameters.Add(objParam)

                objTemp = objCmd.ExecuteScalar()
                If TypeOf objTemp Is DBNull Then
                    intRowCount = 0
                Else
                    intRowCount = CType(objTemp, Integer)
                End If

                If intRowCount > 0 Then
                    InventoryRecExists = True
                Else
                    InventoryRecExists = False
                End If

                ContextUtil.SetComplete()

            Catch E As Exception
                InventoryRecExists = False
                ContextUtil.SetAbort()
            Finally
                objCnn.Close()
            End Try


        End Function

        Private Sub UpdateInventoryRecord(ByVal BinNumber As String, _
                    ByVal SKU As String, _
                    ByVal Quantity As Integer)

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim intRowCount As Integer

            Try
                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = "UPDATE Inventory " & _
                    "SET Quantity = Quantity + @Quantity " & _
                    "WHERE BinNumber = @BinNumber AND SKU = @SKU"
                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@Quantity"
                    .SqlDbType = SqlDbType.Int
                    .Direction = ParameterDirection.Input
                    .Value = Quantity
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@BinNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = BinNumber
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@SKU"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = SKU
                End With
                objCmd.Parameters.Add(objParam)

                intRowCount = objCmd.ExecuteNonQuery()
                ContextUtil.SetComplete()

            Catch E As Exception
                ContextUtil.SetAbort()
            Finally
                objCnn.Close()
            End Try
        End Sub

        Private Sub InsertInventoryRecord(ByVal BinNumber As String, _
                ByVal SKU As String, _
                ByVal Quantity As Integer)

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim intRowCount As Integer

            Try
                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = "INSERT INTO Inventory " & _
                    "( BinNumber, SKU, Quantity ) VALUES " & _
                    "( @BinNumber, @SKU,  @Quantity )"
                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@BinNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = BinNumber
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@SKU"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = SKU
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@Quantity"
                    .SqlDbType = SqlDbType.Int
                    .Direction = ParameterDirection.Input
                    .Value = Quantity
                End With
                objCmd.Parameters.Add(objParam)

                intRowCount = objCmd.ExecuteNonQuery()
                ContextUtil.SetComplete()

            Catch E As Exception
                ContextUtil.SetAbort()
            Finally
                objCnn.Close()
            End Try
        End Sub

        Public Function Update(ByVal BinNumber As String, _
                ByVal SKU As String, _
                ByVal Quantity As Integer) _
                As Boolean _
                Implements IUpdateInventory.Update

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim strStockingLocation As String
            Dim intRowsAffected As Integer

            Try
                If InventoryRecExists(SKU, BinNumber) Then
                    UpdateInventoryRecord(BinNumber, SKU, Quantity)
                Else
                    InsertInventoryRecord(BinNumber, SKU, Quantity)
                End If

                Update = True
                ContextUtil.SetComplete()

            Catch E As Exception
                Update = False
                ContextUtil.SetAbort()
            End Try

        End Function
    End Class
End Namespace
