Imports System
Imports System.Reflection
Imports System.EnterpriseServices
Imports System.Data
Imports System.Data.SqlClient


Namespace Supermarket

    Public Interface IReceiveOrder
        Function GetNextLineItem(ByVal OrderNumber As String, _
                        ByRef SKU As String) As Integer
        Function Receive(ByVal OrderNumber As String, _
                        ByVal SKU As String, _
                        ByVal LineNumber As Integer, _
                        ByVal QuantityReceived As Integer) _
                        As Boolean
    End Interface

    <ConstructionEnabled([Default]:="Default Construction String"), _
    Transaction(TransactionOption.Required)> _
    Public Class ReceiveOrder
        Inherits ServicedComponent
        Implements IReceiveOrder

        Public Sub New()

        End Sub

        Private Sub UpdateOrderDeatils(ByVal OrderNumber As String, _
                        ByVal LineNumber As Integer, _
                        ByVal QuantityReceived As Integer)

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim objSQLDr As SqlDataReader
            Dim intRowsAffected As Integer

            Try
                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = "UPDATE OrderDetails " & _
                    "SET QuantityReceived = QuantityReceived + @QuantityReceived " & _
                    "WHERE OrderNumber = @OrderNumber AND LineItemNumber = @LineNumber"
                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@QuantityReceived"
                    .SqlDbType = SqlDbType.Int
                    .Direction = ParameterDirection.Input
                    .Value = QuantityReceived
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@OrderNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = OrderNumber
                End With
                objCmd.Parameters.Add(objParam)

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@LineNumber"
                    .SqlDbType = SqlDbType.Int
                    .Direction = ParameterDirection.Input
                    .Value = LineNumber
                End With
                objCmd.Parameters.Add(objParam)

                intRowsAffected = objCmd.ExecuteNonQuery()

                ContextUtil.SetComplete()

            Catch E As Exception
                ContextUtil.SetAbort()
            Finally
                objCnn.Close()
            End Try

        End Sub

        Public Function GetNextLineItem(ByVal OrderNumber As String, _
                            ByRef SKU As String) _
                            As Integer Implements _
                            IReceiveOrder.GetNextLineItem

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim objSQLDr As SqlDataReader

            Try

                objCnn = New SqlConnection()
                objCnn.ConnectionString = _
                    "Initial Catalog=Supermarket;Data Source=localhost;uid=sa;pwd="
                objCnn.Open()
                objCmd = objCnn.CreateCommand()

                objCmd.CommandText = "SELECT MAX(LineItemNumber), " & _
                    "SKU FROM OrderDetails od, Orders o " & _
                    "WHERE od.OrderNumber = @OrderNumber AND " & _
                    "o.OrderReceived = 0 AND " & _
                    "o.OrderNumber = od.OrderNumber GROUP BY SKU"

                objParam = New SqlParameter()
                With objParam
                    .ParameterName = "@OrderNumber"
                    .SqlDbType = SqlDbType.VarChar
                    .Direction = ParameterDirection.Input
                    .Value = OrderNumber
                End With
                objCmd.Parameters.Add(objParam)

                objSQLDr = objCmd.ExecuteReader()
                objSQLDr.Read()

                If Not objSQLDr.IsDBNull(0) Then
                    GetNextLineItem = objSQLDr.GetInt32(0)
                    SKU = objSQLDr.GetString(1)
                Else
                    GetNextLineItem = -1
                    SKU = ""
                End If

                ContextUtil.SetComplete()
                objSQLDr.Close()

            Catch E As Exception
                GetNextLineItem = -1
                SKU = ""
                ContextUtil.SetAbort()
            Finally
                objCnn.Close()
            End Try

        End Function

        Public Function Receive(ByVal OrderNumber As String, _
                        ByVal SKU As String, _
                        ByVal LineNumber As Integer, _
                        ByVal QuantityReceived As Integer) _
                        As Boolean _
                        Implements IReceiveOrder.Receive

            Dim objCnn As SqlConnection
            Dim objCmd As SqlCommand
            Dim objParam As SqlParameter
            Dim objInvUpdate As IUpdateInventory
            Dim strBinNumber As String

            Try
                UpdateOrderDeatils(OrderNumber, _
                        LineNumber, _
                        QuantityReceived)

                objInvUpdate = New UpdateInventory()
                strBinNumber = objInvUpdate.GetStockingLocation(SKU)
                If objInvUpdate.Update(strBinNumber, _
                        SKU, QuantityReceived) Then
                    Receive = True
                    ContextUtil.SetComplete()
                Else
                    Receive = False
                    ContextUtil.SetAbort()
                End If

            Catch E As Exception
                Receive = False
                ContextUtil.SetAbort()
            End Try
        End Function

    End Class
End Namespace
