Imports System
Imports System.IO
Imports Microsoft.VisualBasic

Module Module1

    Public Enum ID3Offsets
        intArtistOffset = 34
        intTitleOffset = 4
        intAlbumOffset = 64
        intYearOffset = 94
        intCommentOffset = 98
        intGenreCodeOffset = 128
    End Enum 

    Public Class MP3ID3v1

        Private m_objFS As FileStream
        Public m_strTitle As String
        Public m_strArtist As String
        Public m_strAlbum As String
        Public m_strYear As String
        Public m_strComment As String
        Public m_strRawID3 As String
        Public m_strGenreCode As String

        Public Sub New(ByVal Filename As String)
            RefreshID3Info(Filename)
        End Sub

        Public Sub RefreshID3Info(ByVal FileName As String)
            Dim bytTag(128) As Byte

            Try
                m_objFS = New FileStream(FileName, _
                        FileMode.Open, _
                        FileAccess.ReadWrite)
                m_objFS.Seek(m_objFS.Length - 128, _
				SeekOrigin.Begin)
                m_objFS.Read(bytTag, 0, 128)		
                m_strRawID3 = BytesToString(bytTag)	

            	If Left(m_strRawID3, 3) = "TAG" Then		
                	m_strTitle = Trim(Mid(m_strRawID3, _
                        	ID3Offsets.intTitleOffset, 30))
               		m_strArtist = Trim(Mid(m_strRawID3, _
                        	ID3Offsets.intArtistOffset, 30))
                	m_strAlbum = Trim(Mid(m_strRawID3, _
                        	ID3Offsets.intAlbumOffset, 30))
                	m_strYear = Trim(Mid(m_strRawID3, _
                        	ID3Offsets.intYearOffset, 4))
                	m_strComment = Trim(Mid(m_strRawID3, _
                        	ID3Offsets.intCommentOffset, 30))
                	m_strGenreCode = Asc(Mid(m_strRawID3, _
                        	ID3Offsets.intGenreCodeOffset, 1))
            	Else
                	m_strRawID3 = Space(128)
            	End If

            Catch E As Exception
                Console.WriteLine("Error: " & E.Message)
	    End Try
        End Sub

        Private Function BytesToString( _
                ByVal bytBytes() As Byte) As String

            Dim byt As Byte
            Dim strReturnString As String
            For Each byt In bytBytes
                If byt <> 0 Then
                    strReturnString &= ChrW(byt)
                Else
                    strReturnString &= " "
                End If
            Next
            BytesToString = strReturnString
        End Function

	Public Sub WriteID3Info( _
			Optional ByVal Title As String = "", _
			Optional ByVal Artist As String = "", _
			Optional ByVal Album As String = "", _
			Optional ByVal Year As String = "", _
			Optional ByVal Comment As String = "", _
			Optional ByVal GenreCode As Integer = 0)

	Dim strNewRawID3 As String = "TAG"

	If Title = "" Then				
		strNewRawID3 &= m_strTitle.PadRight(30)	
	Else
		strNewRawID3 &= Title.PadRight(30)	
	End If

	If Artist = "" Then
		strNewRawID3 &= m_strArtist.PadRight(30)	
	Else
		strNewRawID3 &= Artist.PadRight(30)	
	End If

	If Album = "" Then
		strNewRawID3 &= m_strAlbum.PadRight(30)	
	Else
		strNewRawID3 &= Album.PadRight(30)	
	End If

	If Year = "" Then
		strNewRawID3 &= m_strYear.PadRight(4)	
	Else
		strNewRawID3 &= Year.PadRight(4)		
	End If

	If Comment = "" Then
		strNewRawID3 &= m_strComment.PadRight(30) 
	Else
		strNewRawID3 &= Comment.PadRight(30)	
	End If

	If GenreCode = 0 Then
		strNewRawID3 &= Chr(CInt(m_strGenreCode)) 
	Else
		strNewRawID3 &= Chr(GenreCode)		
	End If

	Try
		With m_objFS
			.Seek(m_objFS.Length - 128, SeekOrigin.Begin)	
			.Write(StringToBytes(strNewRawID3), 0, 127)		
			.Flush()						
			.Close()						
		End With

	Catch E As Exception
		Console.WriteLine("Error writing ID3 info: " & E.Message)
	End Try

End Sub

Private Function StringToBytes( _
			ByVal strString As String) As Byte()

	Dim ch As Char
	Dim intIndex As Integer
	Dim bytReturnArray(128) As Byte

	For intIndex = 0 To 127
		bytReturnArray(intIndex) = _
			Asc(strString.Chars(intIndex))
	Next

	StringToBytes = bytReturnArray

End Function


    End Class


    Sub Main()
        
	Dim objMP3Info As New MP3ID3v1("demo.mp3")

        With objMP3Info
       		Console.WriteLine("Artist: {0}", .m_strArtist)
    		Console.WriteLine("Title: {0}", .m_strTitle)
        	Console.WriteLine("Album: {0}", .m_strAlbum)
        	Console.WriteLine("Year: {0}", .m_strYear)
        	Console.WriteLine("Comment: {0}", .m_strComment)
        	Console.WriteLine("Genre Code: {0}", .m_strGenreCode)
        End With

	objMP3Info.WriteID3Info( "Sample MP3", "Matt J. Crouch","","2001","",0)
    End Sub

End Module
